@@ -1,5 +1,6 @@
 
 #include "config.h"
+#include "libssh/libssh.h"
 
 #define LIBSSH_STATIC
 
@@ -164,7 +165,7 @@ static void torture_pki_rsa_import_privkey_base64_NULL_key(void **state)
                                        NULL,
                                        NULL,
                                        NULL);
-    assert_true(rc == -1);
+    assert_int_equal(rc, -1);
 
 }
 
@@ -178,16 +179,18 @@ static void torture_pki_rsa_import_privkey_base64_NULL_str(void **state)
 
     /* test if it returns -1 if key_str is NULL */
     rc = ssh_pki_import_privkey_base64(NULL, passphrase, NULL, NULL, &key);
-    assert_true(rc == -1);
+    assert_int_equal(rc, -1);
 
     SSH_KEY_FREE(key);
 }
 
-static void torture_pki_rsa_import_privkey_base64(void **state)
+static void
+torture_pki_rsa_import_export_privkey_base64_format(void **state,
+                                                    enum ssh_file_format_e format)
 {
     int rc;
-    char *key_str = NULL;
-    ssh_key key = NULL;
+    char *key_str = NULL, *new_key_str = NULL;
+    ssh_key key = NULL, new_key = NULL;
     const char *passphrase = torture_get_testkey_passphrase();
     enum ssh_keytypes_e type;
 
@@ -196,21 +199,62 @@ static void torture_pki_rsa_import_privkey_base64(void **state)
     key_str = torture_pki_read_file(LIBSSH_RSA_TESTKEY);
     assert_non_null(key_str);
 
+    /* Import test key */
     rc = ssh_pki_import_privkey_base64(key_str, passphrase, NULL, NULL, &key);
-    assert_true(rc == 0);
+    assert_return_code(rc, errno);
     assert_non_null(key);
 
     type = ssh_key_type(key);
-    assert_true(type == SSH_KEYTYPE_RSA);
+    assert_int_equal(type, SSH_KEYTYPE_RSA);
 
     rc = ssh_key_is_private(key);
-    assert_true(rc == 1);
+    assert_int_equal(rc, 1);
 
     rc = ssh_key_is_public(key);
-    assert_true(rc == 1);
+    assert_int_equal(rc, 1);
+
+    /* Export */
+    rc = ssh_pki_export_privkey_base64_format(key,
+                                              passphrase,
+                                              NULL,
+                                              NULL,
+                                              &new_key_str,
+                                              format);
+    assert_int_equal(rc, SSH_OK);
+    assert_non_null(new_key_str);
+
+    /* and import again */
+    rc = ssh_pki_import_privkey_base64(new_key_str,
+                                       passphrase,
+                                       NULL,
+                                       NULL,
+                                       &new_key);
+    assert_int_equal(rc, 0);
+    assert_non_null(new_key);
+
+    type = ssh_key_type(new_key);
+    assert_int_equal(type, SSH_KEYTYPE_RSA);
+
+    rc = ssh_key_is_private(new_key);
+    assert_int_equal(rc, 1);
+
+    rc = ssh_key_is_public(new_key);
+    assert_int_equal(rc, 1);
+
+    rc = ssh_key_cmp(key, new_key, SSH_KEY_CMP_PRIVATE|SSH_KEY_CMP_PUBLIC);
+    assert_int_equal(rc, 0);
 
     free(key_str);
+    free(new_key_str);
     SSH_KEY_FREE(key);
+    SSH_KEY_FREE(new_key);
+}
+
+static void
+torture_pki_rsa_import_export_privkey_base64(void **state)
+{
+    torture_pki_rsa_import_export_privkey_base64_format(state,
+                                                        SSH_FILE_FORMAT_DEFAULT);
 }
 
 static void torture_pki_rsa_import_privkey_base64_comment(void **state)
@@ -234,17 +278,17 @@ static void torture_pki_rsa_import_privkey_base64_comment(void **state)
     assert_int_equal(rc, file_str_len - 1);
 
     rc = ssh_pki_import_privkey_base64(file_str, passphrase, NULL, NULL, &key);
-    assert_true(rc == 0);
+    assert_return_code(rc, errno);
     assert_non_null(key);
 
     type = ssh_key_type(key);
-    assert_true(type == SSH_KEYTYPE_RSA);
+    assert_int_equal(type, SSH_KEYTYPE_RSA);
 
     rc = ssh_key_is_private(key);
-    assert_true(rc == 1);
+    assert_int_equal(rc, 1);
 
     rc = ssh_key_is_public(key);
-    assert_true(rc == 1);
+    assert_int_equal(rc, 1);
 
     free(key_str);
     free(file_str);
@@ -272,17 +316,17 @@ static void torture_pki_rsa_import_privkey_base64_whitespace(void **state)
     assert_int_equal(rc, file_str_len - 1);
 
     rc = ssh_pki_import_privkey_base64(file_str, passphrase, NULL, NULL, &key);
-    assert_true(rc == 0);
+    assert_return_code(rc, errno);
     assert_non_null(key);
 
     type = ssh_key_type(key);
-    assert_true(type == SSH_KEYTYPE_RSA);
+    assert_int_equal(type, SSH_KEYTYPE_RSA);
 
     rc = ssh_key_is_private(key);
-    assert_true(rc == 1);
+    assert_int_equal(rc, 1);
 
     rc = ssh_key_is_public(key);
-    assert_true(rc == 1);
+    assert_int_equal(rc, 1);
 
     free(key_str);
     free(file_str);
@@ -303,14 +347,14 @@ static void torture_pki_rsa_publickey_from_privatekey(void **state)
                                        NULL,
                                        NULL,
                                        &key);
-    assert_true(rc == 0);
+    assert_return_code(rc, errno);
     assert_non_null(key);
 
     rc = ssh_key_is_private(key);
-    assert_true(rc == 1);
+    assert_int_equal(rc, 1);
 
     rc = ssh_pki_export_privkey_to_pubkey(key, &pubkey);
-    assert_true(rc == SSH_OK);
+    assert_return_code(rc, errno);
     assert_non_null(pubkey);
 
     SSH_KEY_FREE(key);
@@ -329,48 +373,87 @@ static void torture_pki_rsa_copy_cert_to_privkey(void **state)
     ssh_key pubkey = NULL;
     ssh_key privkey = NULL;
     ssh_key cert = NULL;
+    enum ssh_keytypes_e type;
 
-    (void) state; /* unused */
+    (void)state; /* unused */
+
+    /* Importing public key as cert should fail */
+    rc = ssh_pki_import_cert_file(LIBSSH_RSA_TESTKEY ".pub", &cert);
+    assert_int_equal(rc, SSH_ERROR);
+    assert_null(cert);
 
     rc = ssh_pki_import_cert_file(LIBSSH_RSA_TESTKEY "-cert.pub", &cert);
-    assert_true(rc == SSH_OK);
+    assert_return_code(rc, errno);
     assert_non_null(cert);
 
     rc = ssh_pki_import_pubkey_file(LIBSSH_RSA_TESTKEY ".pub", &pubkey);
-    assert_true(rc == SSH_OK);
+    assert_return_code(rc, errno);
     assert_non_null(pubkey);
 
+    type = ssh_key_type(cert);
+    assert_true(type == SSH_KEYTYPE_RSA_CERT01);
+
+    rc = ssh_key_is_public(cert);
+    assert_int_equal(rc, 1);
+
+    /* Import matching private key file and verify the pubkey matches */
     rc = ssh_pki_import_privkey_base64(torture_get_testkey(SSH_KEYTYPE_RSA, 0),
                                        passphrase,
                                        NULL,
                                        NULL,
                                        &privkey);
-    assert_true(rc == SSH_OK);
+    assert_return_code(rc, errno);
     assert_non_null(privkey);
 
+    type = ssh_key_type(privkey);
+    assert_true(type == SSH_KEYTYPE_RSA);
+
     /* Basic sanity. */
     rc = ssh_pki_copy_cert_to_privkey(NULL, privkey);
-    assert_true(rc == SSH_ERROR);
+    assert_int_equal(rc, SSH_ERROR);
 
     rc = ssh_pki_copy_cert_to_privkey(pubkey, NULL);
-    assert_true(rc == SSH_ERROR);
+    assert_int_equal(rc, SSH_ERROR);
 
     /* A public key doesn't have a cert, copy should fail. */
     assert_null(pubkey->cert);
     rc = ssh_pki_copy_cert_to_privkey(pubkey, privkey);
-    assert_true(rc == SSH_ERROR);
+    assert_int_equal(rc, SSH_ERROR);
 
     /* Copying the cert to non-cert keys should work fine. */
     rc = ssh_pki_copy_cert_to_privkey(cert, pubkey);
-    assert_true(rc == SSH_OK);
+    assert_return_code(rc, errno);
     assert_non_null(pubkey->cert);
     rc = ssh_pki_copy_cert_to_privkey(cert, privkey);
-    assert_true(rc == SSH_OK);
+    assert_return_code(rc, errno);
     assert_non_null(privkey->cert);
+    assert_true(privkey->cert_type == SSH_KEYTYPE_RSA_CERT01);
+
+    assert_int_equal(ssh_key_cmp(privkey, cert, SSH_KEY_CMP_PUBLIC), 0);
+    assert_int_equal(ssh_key_cmp(cert, privkey, SSH_KEY_CMP_PUBLIC), 0);
 
     /* The private key's cert is already set, another copy should fail. */
     rc = ssh_pki_copy_cert_to_privkey(cert, privkey);
-    assert_true(rc == SSH_ERROR);
+    assert_int_equal(rc, SSH_ERROR);
+
+    SSH_KEY_FREE(privkey);
+    SSH_KEY_FREE(pubkey);
+
+    /* Generate different key and try to assign it this certificate */
+    rc = ssh_pki_generate(SSH_KEYTYPE_RSA, 2048, &privkey);
+    assert_return_code(rc, errno);
+    assert_non_null(privkey);
+    rc = ssh_pki_export_privkey_to_pubkey(privkey, &pubkey);
+    assert_return_code(rc, errno);
+    assert_non_null(pubkey);
+
+    rc = ssh_pki_copy_cert_to_privkey(cert, privkey);
+    assert_int_equal(rc, SSH_ERROR);
+    rc = ssh_pki_copy_cert_to_privkey(cert, pubkey);
+    assert_int_equal(rc, SSH_ERROR);
+
+    assert_int_equal(ssh_key_cmp(privkey, cert, SSH_KEY_CMP_PUBLIC), 1);
+    assert_int_equal(ssh_key_cmp(cert, privkey, SSH_KEY_CMP_PUBLIC), 1);
 
     SSH_KEY_FREE(cert);
     SSH_KEY_FREE(privkey);
@@ -385,14 +468,14 @@ static void torture_pki_rsa_import_cert_file(void **state) {
     (void) state; /* unused */
 
     rc = ssh_pki_import_cert_file(LIBSSH_RSA_TESTKEY "-cert.pub", &cert);
-    assert_true(rc == 0);
+    assert_return_code(rc, errno);
     assert_non_null(cert);
 
     type = ssh_key_type(cert);
-    assert_true(type == SSH_KEYTYPE_RSA_CERT01);
+    assert_int_equal(type, SSH_KEYTYPE_RSA_CERT01);
 
     rc = ssh_key_is_public(cert);
-    assert_true(rc == 1);
+    assert_int_equal(rc, 1);
 
     SSH_KEY_FREE(cert);
 }
@@ -417,7 +500,7 @@ static void torture_pki_rsa_publickey_base64(void **state)
     }
 
     type = ssh_key_type_from_name(q);
-    assert_true(type == SSH_KEYTYPE_RSA);
+    assert_int_equal(type, SSH_KEYTYPE_RSA);
 
     q = ++p;
     while (p != NULL && *p != '\0' && *p != ' ') p++;
@@ -426,11 +509,11 @@ static void torture_pki_rsa_publickey_base64(void **state)
     }
 
     rc = ssh_pki_import_pubkey_base64(q, type, &key);
-    assert_true(rc == 0);
+    assert_return_code(rc, errno);
     assert_non_null(key);
 
     rc = ssh_pki_export_pubkey_base64(key, &b64_key);
-    assert_true(rc == 0);
+    assert_return_code(rc, errno);
     assert_non_null(b64_key);
 
     assert_string_equal(q, b64_key);
@@ -457,20 +540,20 @@ static void torture_pki_rsa_generate_pubkey_from_privkey(void **state) {
                                      NULL,
                                      NULL,
                                      &privkey);
-    assert_true(rc == 0);
+    assert_return_code(rc, errno);
     assert_non_null(privkey);
 
     rc = ssh_pki_export_privkey_to_pubkey(privkey, &pubkey);
-    assert_true(rc == SSH_OK);
+    assert_return_code(rc, errno);
     assert_non_null(pubkey);
 
     rc = ssh_pki_export_pubkey_file(pubkey, LIBSSH_RSA_TESTKEY ".pub");
-    assert_true(rc == 0);
+    assert_return_code(rc, errno);
 
     rc = torture_read_one_line(LIBSSH_RSA_TESTKEY ".pub",
                                pubkey_generated,
                                sizeof(pubkey_generated));
-    assert_true(rc == 0);
+    assert_return_code(rc, errno);
 
     len = torture_pubkey_len(torture_get_testkey_pub(SSH_KEYTYPE_RSA));
     assert_memory_equal(torture_get_testkey_pub(SSH_KEYTYPE_RSA),
@@ -494,11 +577,11 @@ static void torture_pki_rsa_duplicate_key(void **state)
     (void) state;
 
     rc = ssh_pki_import_pubkey_file(LIBSSH_RSA_TESTKEY ".pub", &pubkey);
-    assert_true(rc == 0);
+    assert_return_code(rc, errno);
     assert_non_null(pubkey);
 
     rc = ssh_pki_export_pubkey_base64(pubkey, &b64_key);
-    assert_true(rc == 0);
+    assert_return_code(rc, errno);
     assert_non_null(b64_key);
 
     rc = ssh_pki_import_privkey_file(LIBSSH_RSA_TESTKEY,
@@ -506,27 +589,27 @@ static void torture_pki_rsa_duplicate_key(void **state)
                                      NULL,
                                      NULL,
                                      &privkey);
-    assert_true(rc == 0);
+    assert_return_code(rc, errno);
     assert_non_null(privkey);
 
     privkey_dup = ssh_key_dup(privkey);
     assert_non_null(privkey_dup);
 
     rc = ssh_pki_export_privkey_to_pubkey(privkey, &pubkey_dup);
-    assert_true(rc == SSH_OK);
+    assert_return_code(rc, errno);
     assert_non_null(pubkey_dup);
 
     rc = ssh_pki_export_pubkey_base64(pubkey_dup, &b64_key_gen);
-    assert_true(rc == 0);
+    assert_return_code(rc, errno);
     assert_non_null(b64_key_gen);
 
     assert_string_equal(b64_key, b64_key_gen);
 
     rc = ssh_key_cmp(privkey, privkey_dup, SSH_KEY_CMP_PRIVATE);
-    assert_true(rc == 0);
+    assert_return_code(rc, errno);
 
     rc = ssh_key_cmp(pubkey, pubkey_dup, SSH_KEY_CMP_PUBLIC);
-    assert_true(rc == 0);
+    assert_return_code(rc, errno);
 
     SSH_KEY_FREE(pubkey);
     SSH_KEY_FREE(pubkey_dup);
@@ -550,7 +633,7 @@ static void torture_pki_rsa_generate_key(void **state)
 
     if (!ssh_fips_mode()) {
         rc = ssh_pki_generate(SSH_KEYTYPE_RSA, 1024, &key);
-        assert_true(rc == SSH_OK);
+        assert_return_code(rc, errno);
         assert_non_null(key);
         rc = ssh_pki_export_privkey_to_pubkey(key, &pubkey);
         assert_int_equal(rc, SSH_OK);
@@ -558,7 +641,7 @@ static void torture_pki_rsa_generate_key(void **state)
         sign = pki_do_sign(key, INPUT, sizeof(INPUT), SSH_DIGEST_SHA256);
         assert_non_null(sign);
         rc = ssh_pki_signature_verify(session, sign, pubkey, INPUT, sizeof(INPUT));
-        assert_true(rc == SSH_OK);
+        assert_return_code(rc, errno);
         ssh_signature_free(sign);
         SSH_KEY_FREE(key);
         SSH_KEY_FREE(pubkey);
@@ -567,7 +650,7 @@ static void torture_pki_rsa_generate_key(void **state)
     }
 
     rc = ssh_pki_generate(SSH_KEYTYPE_RSA, 2048, &key);
-    assert_true(rc == SSH_OK);
+    assert_return_code(rc, errno);
     assert_non_null(key);
     rc = ssh_pki_export_privkey_to_pubkey(key, &pubkey);
     assert_int_equal(rc, SSH_OK);
@@ -575,7 +658,7 @@ static void torture_pki_rsa_generate_key(void **state)
     sign = pki_do_sign(key, INPUT, sizeof(INPUT), SSH_DIGEST_SHA256);
     assert_non_null(sign);
     rc = ssh_pki_signature_verify(session, sign, pubkey, INPUT, sizeof(INPUT));
-    assert_true(rc == SSH_OK);
+    assert_return_code(rc, errno);
     ssh_signature_free(sign);
     SSH_KEY_FREE(key);
     SSH_KEY_FREE(pubkey);
@@ -583,7 +666,7 @@ static void torture_pki_rsa_generate_key(void **state)
     pubkey = NULL;
 
     rc = ssh_pki_generate(SSH_KEYTYPE_RSA, 4096, &key);
-    assert_true(rc == SSH_OK);
+    assert_return_code(rc, errno);
     assert_non_null(key);
     rc = ssh_pki_export_privkey_to_pubkey(key, &pubkey);
     assert_int_equal(rc, SSH_OK);
@@ -591,7 +674,7 @@ static void torture_pki_rsa_generate_key(void **state)
     sign = pki_do_sign(key, INPUT, sizeof(INPUT), SSH_DIGEST_SHA256);
     assert_non_null(sign);
     rc = ssh_pki_signature_verify(session, sign, pubkey, INPUT, sizeof(INPUT));
-    assert_true(rc == SSH_OK);
+    assert_return_code(rc, errno);
     ssh_signature_free(sign);
     SSH_KEY_FREE(key);
     SSH_KEY_FREE(pubkey);
@@ -613,11 +696,11 @@ static void torture_pki_rsa_sha2(void **state)
 
     /* Setup */
     rc  = ssh_pki_import_privkey_file(LIBSSH_RSA_TESTKEY, NULL, NULL, NULL, &key);
-    assert_true(rc == SSH_OK);
+    assert_return_code(rc, errno);
     assert_non_null(key);
 
     rc  = ssh_pki_import_cert_file(LIBSSH_RSA_TESTKEY "-cert.pub", &cert);
-    assert_true(rc == SSH_OK);
+    assert_return_code(rc, errno);
     assert_non_null(cert);
 
     /* Get the public key to verify signature */
@@ -680,7 +763,7 @@ static void torture_pki_rsa_key_size(void **state)
     (void) state;
 
     rc = ssh_pki_generate(SSH_KEYTYPE_RSA, 2048, &key);
-    assert_true(rc == SSH_OK);
+    assert_return_code(rc, errno);
     assert_non_null(key);
     rc = ssh_pki_export_privkey_to_pubkey(key, &pubkey);
     assert_int_equal(rc, SSH_OK);
@@ -690,13 +773,13 @@ static void torture_pki_rsa_key_size(void **state)
     rc = ssh_pki_signature_verify(session, sign, pubkey, INPUT, sizeof(INPUT));
     assert_ssh_return_code(session, rc);
 
-    /* Set the minumum RSA key size to 4k */
+    /* Set the minimum RSA key size to 4k */
     rc = ssh_options_set(session, SSH_OPTIONS_RSA_MIN_SIZE, &length);
     assert_ssh_return_code(session, rc);
 
     /* the verification should fail now */
     rc = ssh_pki_signature_verify(session, sign, pubkey, INPUT, sizeof(INPUT));
-    assert_true(rc == SSH_ERROR);
+    assert_int_equal(rc, SSH_ERROR);
 
     ssh_signature_free(sign);
     SSH_KEY_FREE(key);
@@ -807,8 +890,9 @@ static void torture_pki_fail_sign_with_incompatible_hash(void **state)
     SSH_KEY_FREE(key);
 }
 
-#ifdef HAVE_LIBCRYPTO
-static void torture_pki_rsa_write_privkey(void **state)
+static void
+torture_pki_rsa_write_privkey_format(void **state,
+                                     enum ssh_file_format_e format)
 {
     ssh_key origkey = NULL;
     ssh_key privkey = NULL;
@@ -821,28 +905,29 @@ static void torture_pki_rsa_write_privkey(void **state)
                                      NULL,
                                      NULL,
                                      &origkey);
-    assert_true(rc == 0);
+    assert_return_code(rc, errno);
     assert_non_null(origkey);
 
     unlink(LIBSSH_RSA_TESTKEY);
 
-    rc = ssh_pki_export_privkey_file(origkey,
-                                     NULL,
-                                     NULL,
-                                     NULL,
-                                     LIBSSH_RSA_TESTKEY);
-    assert_true(rc == 0);
+    rc = ssh_pki_export_privkey_file_format(origkey,
+                                            NULL,
+                                            NULL,
+                                            NULL,
+                                            LIBSSH_RSA_TESTKEY,
+                                            format);
+    assert_return_code(rc, errno);
 
     rc = ssh_pki_import_privkey_file(LIBSSH_RSA_TESTKEY,
                                      NULL,
                                      NULL,
                                      NULL,
                                      &privkey);
-    assert_true(rc == 0);
+    assert_return_code(rc, errno);
     assert_non_null(privkey);
 
     rc = ssh_key_cmp(origkey, privkey, SSH_KEY_CMP_PRIVATE);
-    assert_true(rc == 0);
+    assert_return_code(rc, errno);
 
     SSH_KEY_FREE(origkey);
     SSH_KEY_FREE(privkey);
@@ -853,16 +938,17 @@ static void torture_pki_rsa_write_privkey(void **state)
                                      NULL,
                                      NULL,
                                      &origkey);
-    assert_true(rc == 0);
+    assert_return_code(rc, errno);
     assert_non_null(origkey);
 
     unlink(LIBSSH_RSA_TESTKEY_PASSPHRASE);
-    rc = ssh_pki_export_privkey_file(origkey,
-                                     torture_get_testkey_passphrase(),
-                                     NULL,
-                                     NULL,
-                                     LIBSSH_RSA_TESTKEY_PASSPHRASE);
-    assert_true(rc == 0);
+    rc = ssh_pki_export_privkey_file_format(origkey,
+                                            torture_get_testkey_passphrase(),
+                                            NULL,
+                                            NULL,
+                                            LIBSSH_RSA_TESTKEY_PASSPHRASE,
+                                            format);
+    assert_return_code(rc, errno);
 
     /* Test with invalid passphrase */
     rc = ssh_pki_import_privkey_file(LIBSSH_RSA_TESTKEY_PASSPHRASE,
@@ -870,7 +956,7 @@ static void torture_pki_rsa_write_privkey(void **state)
                                      NULL,
                                      NULL,
                                      &privkey);
-    assert_true(rc == SSH_ERROR);
+    assert_int_equal(rc, SSH_ERROR);
     assert_null(privkey);
 
     rc = ssh_pki_import_privkey_file(LIBSSH_RSA_TESTKEY_PASSPHRASE,
@@ -878,15 +964,47 @@ static void torture_pki_rsa_write_privkey(void **state)
                                      NULL,
                                      NULL,
                                      &privkey);
-    assert_true(rc == 0);
+    assert_return_code(rc, errno);
     assert_non_null(privkey);
 
     rc = ssh_key_cmp(origkey, privkey, SSH_KEY_CMP_PRIVATE);
-    assert_true(rc == 0);
+    assert_return_code(rc, errno);
 
     SSH_KEY_FREE(origkey);
     SSH_KEY_FREE(privkey);
 }
+
+static void
+torture_pki_rsa_write_privkey(void **state)
+{
+    torture_pki_rsa_write_privkey_format(state, SSH_FILE_FORMAT_DEFAULT);
+}
+
+#if defined(HAVE_LIBCRYPTO)
+static void
+torture_pki_rsa_write_privkey_pem(void **state)
+{
+    torture_pki_rsa_write_privkey_format(state, SSH_FILE_FORMAT_PEM);
+}
+
+static void
+torture_pki_rsa_write_privkey_openssh(void **state)
+{
+    torture_pki_rsa_write_privkey_format(state, SSH_FILE_FORMAT_OPENSSH);
+}
+
+static void
+torture_pki_rsa_import_export_privkey_base64_pem(void **state)
+{
+    torture_pki_rsa_import_export_privkey_base64_format(state,
+                                                        SSH_FILE_FORMAT_PEM);
+}
+static void
+torture_pki_rsa_import_export_privkey_base64_openssh(void **state)
+{
+    torture_pki_rsa_import_export_privkey_base64_format(state,
+                                                        SSH_FILE_FORMAT_OPENSSH);
+}
 #endif /* HAVE_LIBCRYPTO */
 
 static void torture_pki_rsa_import_privkey_base64_passphrase(void **state)
@@ -907,7 +1025,7 @@ static void torture_pki_rsa_import_privkey_base64_passphrase(void **state)
     assert_non_null(key);
 
     rc = ssh_key_is_private(key);
-    assert_true(rc == 1);
+    assert_int_equal(rc, 1);
 
     SSH_KEY_FREE(key);
 
@@ -917,7 +1035,7 @@ static void torture_pki_rsa_import_privkey_base64_passphrase(void **state)
                                        NULL,
                                        NULL,
                                        &key);
-    assert_true(rc == -1);
+    assert_int_equal(rc, -1);
     SSH_KEY_FREE(key);
 
 #ifndef HAVE_LIBCRYPTO
@@ -928,7 +1046,7 @@ static void torture_pki_rsa_import_privkey_base64_passphrase(void **state)
                                        NULL,
                                        NULL,
                                        &key);
-    assert_true(rc == -1);
+    assert_int_equal(rc, -1);
     SSH_KEY_FREE(key);
 #endif
 }
@@ -955,7 +1073,7 @@ torture_pki_rsa_import_openssh_privkey_base64_passphrase(void **state)
     assert_non_null(key);
 
     rc = ssh_key_is_private(key);
-    assert_true(rc == 1);
+    assert_int_equal(rc, 1);
 
     SSH_KEY_FREE(key);
 
@@ -965,7 +1083,7 @@ torture_pki_rsa_import_openssh_privkey_base64_passphrase(void **state)
                                        NULL,
                                        NULL,
                                        &key);
-    assert_true(rc == -1);
+    assert_int_equal(rc, -1);
     SSH_KEY_FREE(key);
 
     /* test if it returns -1 if passphrase is NULL */
@@ -975,7 +1093,7 @@ torture_pki_rsa_import_openssh_privkey_base64_passphrase(void **state)
                                        NULL,
                                        NULL,
                                        &key);
-    assert_true(rc == -1);
+    assert_int_equal(rc, -1);
     SSH_KEY_FREE(key);
 }
 
@@ -994,7 +1112,7 @@ int torture_run_tests(void) {
         cmocka_unit_test_setup_teardown(torture_pki_rsa_import_privkey_base64_NULL_str,
                                         setup_rsa_key,
                                         teardown),
-        cmocka_unit_test_setup_teardown(torture_pki_rsa_import_privkey_base64,
+        cmocka_unit_test_setup_teardown(torture_pki_rsa_import_export_privkey_base64,
                                         setup_rsa_key,
                                         teardown),
         cmocka_unit_test_setup_teardown(torture_pki_rsa_import_privkey_base64_comment,
@@ -1003,9 +1121,10 @@ int torture_run_tests(void) {
         cmocka_unit_test_setup_teardown(torture_pki_rsa_import_privkey_base64_whitespace,
                                         setup_rsa_key,
                                         teardown),
-        cmocka_unit_test_setup_teardown(torture_pki_rsa_import_privkey_base64,
-                                        setup_openssh_rsa_key,
-                                        teardown),
+        cmocka_unit_test_setup_teardown(
+            torture_pki_rsa_import_export_privkey_base64,
+            setup_openssh_rsa_key,
+            teardown),
         cmocka_unit_test_setup_teardown(torture_pki_rsa_publickey_from_privatekey,
                                         setup_rsa_key,
                                         teardown),
@@ -1028,10 +1147,24 @@ int torture_run_tests(void) {
                                         teardown),
         cmocka_unit_test(torture_pki_rsa_generate_key),
         cmocka_unit_test(torture_pki_rsa_key_size),
-#if defined(HAVE_LIBCRYPTO)
         cmocka_unit_test_setup_teardown(torture_pki_rsa_write_privkey,
                                         setup_rsa_key,
                                         teardown),
+#if defined(HAVE_LIBCRYPTO)
+        cmocka_unit_test_setup_teardown(torture_pki_rsa_write_privkey_pem,
+                                        setup_rsa_key,
+                                        teardown),
+        cmocka_unit_test_setup_teardown(torture_pki_rsa_write_privkey_openssh,
+                                        setup_rsa_key,
+                                        teardown),
+        cmocka_unit_test_setup_teardown(
+            torture_pki_rsa_import_export_privkey_base64_pem,
+            setup_openssh_rsa_key,
+            teardown),
+        cmocka_unit_test_setup_teardown(
+            torture_pki_rsa_import_export_privkey_base64_openssh,
+            setup_openssh_rsa_key,
+            teardown),
 #endif /* HAVE_LIBCRYPTO */
         cmocka_unit_test(torture_pki_sign_data_rsa),
         cmocka_unit_test(torture_pki_fail_sign_with_incompatible_hash),
